@@ -9,7 +9,7 @@ import os
 import sysconfig
 import uuid
 from pathlib import Path
-from typing import Callable, Dict, List, Sequence, Tuple
+from typing import Any, Callable, Dict, List, Sequence, Tuple
 
 import numpy as np
 
@@ -56,9 +56,12 @@ dtype_dict = {
     "float32": "float",
     "int64": "int64_t",
     "int32": "int32_t",
+    "int16": "int16_t",
     "int8": "int8_t",
     "uint64": "uint64_t",
     "uint32": "uint32_t",
+    "uint16": "uint16_t",
+    "uint8": "uint8_t",
 }
 
 
@@ -226,6 +229,7 @@ class ContextCpu(XContext):
             sources=sources,
             specialize=specialize,
             apply_to_source=apply_to_source,
+            save_source_as=save_source_as,
             extra_compile_args=extra_compile_args,
             extra_link_args=extra_link_args,
             extra_cdef=extra_cdef,
@@ -250,16 +254,22 @@ class ContextCpu(XContext):
         extra_classes=(),
         extra_headers=(),
         compile=True,  # noqa
-    ) -> Dict[str, "KernelCpu"]:
+    ) -> dict[str, "KernelCpu"]:
         # Determine names and paths
         clean_up_so = not module_name
         module_name = module_name or str(uuid.uuid4().hex)
         containing_dir = containing_dir
 
-        classes = classes_from_kernels(kernel_descriptions)
-        classes.update(extra_classes)
+        classes = list(classes_from_kernels(kernel_descriptions))
+        classes += list(extra_classes)
         classes = sort_classes(classes)
 
+        # Update the kernel descriptions with the overriden classes
+        cls_for_name = {cls.__name__: cls for cls in classes}
+        for kernel_name, kernel in kernel_descriptions.items():
+            for arg in kernel.args:
+                arg.atype = cls_for_name.get(arg.atype.__name__, arg.atype)
+
         source, specialized_source = self._build_sources(
             sources=sources,
             classes=classes,
@@ -317,7 +327,11 @@ class ContextCpu(XContext):
             # TODO: find better implementation?
             out_kernels[pyname].description.pyname = pyname
 
-        return out_kernels
+        kernels_with_classes = {
+            (name, tuple(kernel.description.get_overridable_classes())): kernel
+            for name, kernel in out_kernels.items()
+        }
+        return kernels_with_classes
 
     def kernels_from_file(
         self,
@@ -546,7 +560,6 @@ class ContextCpu(XContext):
 
     @property
     def kernels(self):
-
         """
         Dictionary containing all the kernels that have been imported to the context.
         The syntax ``context.kernels.mykernel`` can also be used.
@@ -748,7 +761,6 @@ class KernelCpu:
 
 class FFTCpu(object):
     def __init__(self, data, axes, threads=0):
-
         self.axes = axes
         self.threads = threads
 
