@@ -47,9 +47,14 @@ from ._patch_pyopencl_array import _patch_pyopencl_array
 openclheader: List[SourceType] = [
     """\
 #ifndef XOBJ_STDINT
-typedef long int64_t;
-typedef char int8_t;
-typedef unsigned int uint32_t;
+typedef long           int64_t;
+typedef int            int32_t;
+typedef short          int16_t;
+typedef char           int8_t;
+typedef unsigned long  uint64_t;
+typedef unsigned int   uint32_t;
+typedef unsigned short uint16_t;
+typedef unsigned char  uint8_t;
 #endif
 #ifndef NULL
 #define NULL 0L
@@ -102,7 +107,6 @@ class ContextPyopencl(XContext):
     def __init__(
         self, device=None, patch_pyopencl_array=True, minimum_alignment=None
     ):
-
         """
         Creates a Pyopencl Context object, that allows performing the computations
         on GPUs and CPUs through PyOpenCL.
@@ -184,9 +188,16 @@ class ContextPyopencl(XContext):
         if not compile:
             raise NotImplementedError("compile=False available only on CPU.")
 
-        classes = classes_from_kernels(kernel_descriptions)
-        classes.update(extra_classes)
+        classes = list(classes_from_kernels(kernel_descriptions))
+        classes += list(extra_classes)
         classes = sort_classes(classes)
+
+        # Update the kernel descriptions with the overriden classes
+        cls_for_name = {cls.__name__: cls for cls in classes}
+        for kernel_name, kernel in kernel_descriptions.items():
+            for arg in kernel.args:
+                arg.atype = cls_for_name.get(arg.atype.__name__, arg.atype)
+
         cls_sources = sources_from_classes(classes)
 
         headers = openclheader + list(extra_headers)
@@ -223,10 +234,13 @@ class ContextPyopencl(XContext):
             out_kernels[pyname].source = source
             out_kernels[pyname].specialized_source = specialized_source
 
-        return out_kernels
+        kernels_with_classes = {
+            (name, tuple(kernel.description.get_overridable_classes())): kernel
+            for name, kernel in out_kernels.items()
+        }
+        return kernels_with_classes
 
     def nparray_to_context_array(self, arr):
-
         """
         Copies a numpy array to the device memory.
         Args:
@@ -240,7 +254,6 @@ class ContextPyopencl(XContext):
         return dev_arr
 
     def nparray_from_context_array(self, dev_arr):
-
         """
         Copies an array to the device to a numpy array.
 
@@ -304,7 +317,6 @@ class ContextPyopencl(XContext):
 
     @property
     def kernels(self):
-
         """
         Dictionary containing all the kernels that have been imported to the context.
         The syntax ``context.kernels.mykernel`` can also be used.
@@ -441,7 +453,6 @@ class KernelPyopencl(object):
         context,
         wait_on_call=True,
     ):
-
         self.function = function
         self.description = description
         self.context = context
@@ -505,7 +516,6 @@ class KernelPyopencl(object):
 
 class FFTPyopencl(object):
     def __init__(self, context, data, axes, wait_on_call=True):
-
         self.context = context
         self.axes = axes
         self.wait_on_call = wait_on_call
